from pathlib import Path
import os
import codecs
import lyrebird
import requests
import shutil
from uuid import uuid4
from lyrebird.application import config

BUGIT_STORAGE = lyrebird.get_plugin_storage()
ATTACHMENT_ROOT = Path(BUGIT_STORAGE)/'attachments'

SNAPSHOT_EXPORT_URL = f'http://127.0.0.1:{config.get("mock.port")}/api/snapshot/export/event'
EVENT_EXPORT_URL = f'http://127.0.0.1:{config.get("mock.port")}/api/event/export'


def _check_dir():
    if not ATTACHMENT_ROOT.exists():
        ATTACHMENT_ROOT.mkdir(parents=True, exist_ok=True)


def export_snapshot(snapshot):
    _check_dir()
    res = requests.post(SNAPSHOT_EXPORT_URL, json=snapshot['eventObj'], stream=True)
    # SnapshotId is unique id of snapshot
    id_ = res.headers.get('SnapshotId')
    with codecs.open(str(ATTACHMENT_ROOT / snapshot['name'])+'.lb', 'wb') as f:
        for chunk in res.iter_content():
            f.write(chunk)
    return {
        'id': id_,
        'name': snapshot['name']+'.lb',
        'path': str(ATTACHMENT_ROOT / snapshot['name'])+'.lb'
    }


def export_attachment_file(attachment_obj):
    _check_dir()
    full_name = f'{attachment_obj.get("name", "")}.{attachment_obj.get("attachmentType", "json")}'
    res = requests.post(EVENT_EXPORT_URL, json=attachment_obj['eventObj'], stream=True)
    if res.json().get('code', 1000) == 3000:
        return {
            'name': full_name,
            'path': str(ATTACHMENT_ROOT / full_name)
        }
    with codecs.open(str(ATTACHMENT_ROOT / full_name), 'wb') as f:
        for chunk in res.iter_content():
            f.write(chunk)
    return {
        'id': str(uuid4()),
        'name': full_name,
        'path': str(ATTACHMENT_ROOT / full_name)
    }


def remove_attach():
    if not ATTACHMENT_ROOT.exists():
        pass
    else:
        shutil.rmtree(ATTACHMENT_ROOT)


def rename(attachment_item, new_file_name):
    raw_file_path = Path(attachment_item.get('path'))
    if not raw_file_path.exists():
        raise FileNotFoundError(f'File [{raw_file_path.name}] not exists.')
    new_file_path = raw_file_path.parent.joinpath(new_file_name)
    os.rename(raw_file_path, new_file_path)
    attachment_item['path'] = str(new_file_path)
    attachment_item['name'] = new_file_path.name



import os
import time
import codecs
import shutil
import uuid
import lyrebird
from pathlib import Path
from lyrebird import publish


logger = lyrebird.log.get_logger()

BUGIT_STORAGE = lyrebird.get_plugin_storage()
UPLOAD_ROOT = Path(BUGIT_STORAGE)/'upload'


def get(file_name):
    _check_dir()
    file_path = Path(UPLOAD_ROOT).joinpath(file_name)
    if not file_path.exists():
        return None
    try:
        with codecs.open(str(file_path), 'rb') as f:
            return f.read()
    except Exception:
        return None


def _build_file_name(stream):
    if not stream.filename:
        return str(uuid.uuid4())
    file_name_base, file_name_ext = os.path.splitext(stream.filename)
    file_name = f'{file_name_base}_{int(time.time())}{file_name_ext}'
    return file_name


def add(stream):
    _check_dir()
    file_name = _build_file_name(stream)
    file_path = Path(UPLOAD_ROOT).joinpath(file_name)
    stream.save(str(file_path))
    upload_files = []
    upload_files.append(
        {
            'id': file_name,
            'upload_file': {
                'name': file_name,
                'path': str(file_path)
            }
        }
    )
    publish('upload_files', upload_files)


def delete(file_path):
    file_path = Path(file_path)
    if file_path.parent != UPLOAD_ROOT:
        return
    if file_path.exists() and file_path.is_file():
        file_path.unlink()


def _check_dir():
    if not UPLOAD_ROOT.exists():
        UPLOAD_ROOT.mkdir(parents=True, exist_ok=True)


def reset_dir():
    logger.debug('Upload files reset')
    if UPLOAD_ROOT.exists():
        shutil.rmtree(UPLOAD_ROOT)



import imp
import traceback
from hashlib import md5
from pathlib import Path
from lyrebird import application
from lyrebird import get_logger


logger = get_logger()


def get_workspace():
    bugit_workspace = application.config.get('bugit.workspace')

    if bugit_workspace and Path(bugit_workspace).exists():
        return Path(bugit_workspace)

    ROOT = application._cm.root
    metadata_dir = ROOT/'downloads'/'lyrebird_bugit'
    metadata_dir.mkdir(parents=True, exist_ok=True)
    logger.warning(
        f'Bugit workspace not exists! Create on {metadata_dir._str}')

    return metadata_dir


def template_list():
    template_list = []
    for template_file in get_workspace().iterdir():
        if not template_file.name.endswith('.py'):
            continue
        try:
            logger.debug(f'Load template {template_file}')
            template = imp.load_source(template_file.stem, str(template_file))
            template_check(template)
            template_list.append({
                'name': template.name,
                'path': str(template_file),
                'id': md5(str(template_file).encode()).hexdigest()
            })
            del template
        except Exception:
            logger.error(
                f'Load bug template failed:\nBad template: {template_file}\n{traceback.format_exc()}')
    return template_list


def template_check(template):
    assert hasattr(template, 'name'), "BugIt template should have name attr"
    assert hasattr(template, 'form'), "BugIt template should have form attr"
    assert callable(template.form), "BugIt template should have form function"
    assert hasattr(
        template, 'submit'), "BugIt template should have submit attr"
    assert callable(
        template.submit), "BugIt template should have submit function"


def get_template(file_path):
    template = imp.load_source(Path(file_path).stem, str(file_path))
    return template



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='lyrebird-bugit',
    version='1.12.1',
    packages=['lyrebird_bugit'],
    url='https://github.com/Meituan-Dianping/lyrebird-bugit',
    author='HBQA',
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    zip_safe=False,
    classifiers=(
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
    ),
    entry_points={
        'lyrebird_plugin': [
            'bugit = lyrebird_bugit.manifest'
        ]
    },
    install_requires=[
        "lyrebird>=2.10.3"
    ])



from lyrebird.plugins import manifest
from . import apis
from . import event_handler


manifest(
    id='bugit',
    name='BugIt',
    icon='mdi-bug-outline',
    api=[
        ('/api/template', apis.template, ['GET', 'POST']),
        ('/api/device', apis.device, ['GET']),
        ('/api/issue', apis.issue, ['GET', 'POST']),
        ('/api/take_screenshot/<string:platform>/<string:device_id>', apis.take_screenshot, ['GET']),
        ('/api/attachments', apis.attachments, ['GET', 'POST']),
        ('/api/attachments/<string:attachment_id>', apis.attachments, ['GET', 'PUT', 'DELETE', 'POST']),
        ('/api/cache/<string:template_key>', apis.ui_cache, ['GET', 'POST', 'DELETE'])
    ],
    event=[
        ('android.device', event_handler.on_android_device),
        ('ios.device', event_handler.on_ios_device),
        ('android.screenshot', event_handler.on_android_screenshot),
        ('ios.screenshot', event_handler.on_ios_screenshot),
        ('upload_files', event_handler.on_upload_files)
    ]
)
